using DbContextLib;
using DbDataManager.Service;
using Entities;
using Microsoft.Data.Sqlite;
using Microsoft.EntityFrameworkCore;
using Model.OrderCriteria;

namespace TestEF.Service;

public class TestBlackListDataService
{
    private readonly UserDbContext _dbContext;
    private readonly BlackListDataService _blackListDataService;

    public TestBlackListDataService()
    {
        var connection = new SqliteConnection("DataSource=:memory:");
        connection.Open();
        var options = new DbContextOptionsBuilder<UserDbContext>()
            .UseSqlite(connection)
            .Options;

        _dbContext = new UserDbContext(options);
        _blackListDataService = new BlackListDataService(_dbContext);
    }
    [Fact]
    public void BanUser_Success()
    {
        _dbContext.Users.Add(new UserEntity() { Id = 1, Username = "Test1", Email = "example@email.com", Password = "password", IsAdmin = true });
        _dbContext.Users.Add(new UserEntity() { Id = 2, Username = "Test2", Email = "example@email.com", Password = "password", IsAdmin = false });
        _dbContext.Users.Add(new UserEntity() { Id = 3, Username = "Test3", Email = "example@email.com", Password = "password", IsAdmin = true });
        _dbContext.SaveChanges();
        var banResult = _blackListDataService.BanUser("Test1");
        Assert.True(banResult);
    }
    
    [Fact]
    public void GetNbBannedUsers()
    {
        _dbContext.Users.Add(new UserEntity() { Id = 1, Username = "Test1", Email = "example1@email.com", Password = "password", IsAdmin = true });
        _dbContext.Users.Add(new UserEntity() { Id = 2, Username = "Test2", Email = "example2@email.com", Password = "password", IsAdmin = false });
        _dbContext.Users.Add(new UserEntity() { Id = 3, Username = "Test3", Email = "example3@email.com", Password = "password", IsAdmin = true });
        _dbContext.SaveChanges();
        var banResult1 = _blackListDataService.BanUser("Test1");
        var banResult2 = _blackListDataService.BanUser("Test2");
        var banResult3 = _blackListDataService.BanUser("Test3");
        Assert.True(banResult1);
        Assert.True(banResult2);
        Assert.True(banResult3);
        Assert.Equal(3, _dbContext.BlackLists.Count());
    }
    
    [Fact]
    public void BanUser_Fail()
    {
        _dbContext.Users.Add(new UserEntity() { Id = 1, Username = "Test1", Email = "example@email.com", Password = "password", IsAdmin = true });
        _dbContext.Users.Add(new UserEntity() { Id = 2, Username = "Test2", Email = "example@email.com", Password = "password", IsAdmin = false });
        _dbContext.Users.Add(new UserEntity() { Id = 3, Username = "Test3", Email = "example@email.com", Password = "password", IsAdmin = true });
        _dbContext.SaveChanges();
        var banResult = _blackListDataService.BanUser("Test42");
        Assert.False(banResult);
    }
    
    [Fact]
    public void IsBanned_Success()
    {
        _dbContext.Users.Add(new UserEntity() { Id = 1, Username = "Test1", Email = "example1@email.com", Password = "password", IsAdmin = true });
        _dbContext.Users.Add(new UserEntity() { Id = 2, Username = "Test2", Email = "example@email.com", Password = "password", IsAdmin = false });
        _dbContext.Users.Add(new UserEntity() { Id = 3, Username = "Test3", Email = "example@email.com", Password = "password", IsAdmin = true });
        _dbContext.SaveChanges();
        var banResult = _blackListDataService.BanUser("Test1");
        Assert.True(banResult);
        Assert.NotNull(_blackListDataService.GetUserBannedByEmail("example1@email.com"));
    }
    
    [Fact]
    public void UnbanUser_Success()
    {
        _dbContext.Users.Add(new UserEntity() { Id = 1, Username = "Test1", Email = "example1@email.com", Password = "password", IsAdmin = true });
        _dbContext.Users.Add(new UserEntity() { Id = 2, Username = "Test2", Email = "example@email.com", Password = "password", IsAdmin = false });
        _dbContext.Users.Add(new UserEntity() { Id = 3, Username = "Test3", Email = "example@email.com", Password = "password", IsAdmin = true });
        _dbContext.SaveChanges();
        var banResult = _blackListDataService.BanUser("Test1");
        Assert.True(banResult);
        Assert.True(_blackListDataService.UnbanUser("example1@email.com"));
    }
}